import json
from flask import url_for

from superdesk import get_resource_service


def test_service_get(prodapi_app_with_data):
    """
    Test fetching items using `events_files` service
    :param prodapi_app_with_data: prod api app with filled data
    """

    with prodapi_app_with_data.app_context():
        items_service = get_resource_service("events_files")

        assert len(list(items_service.get(req=None, lookup={}))) == 2


def test_readonly(prodapi_app_with_data, prodapi_app_with_data_client):
    """
    Ensure that `events_files` endpoint is readonly
    :param prodapi_app_with_data: prod api app with filled data
    :param prodapi_app_with_data_client: client for prod api app with filled data
    """

    with prodapi_app_with_data.test_request_context():
        for method, status in (("get", 200), ("post", 405), ("patch", 405), ("put", 405), ("delete", 405)):
            # we send a request
            resp = getattr(prodapi_app_with_data_client, method)(url_for("events_files|resource"))
            # we get a response
            assert resp.status_code == status


def test_excluded_fields(prodapi_app_with_data, prodapi_app_with_data_client):
    """
    Ensure that fields which are listed as `excluded_fields` are not in the response.
    :param prodapi_app_with_data: prod api app with filled data
    :param prodapi_app_with_data_client: client for prod api app with filled data
    """

    excluded_fields = {
        "_etag",
        "_type",
        "_updated",
        "_created",
        "_current_version",
        "_links",
    }

    with prodapi_app_with_data.test_request_context():
        # list
        resp = prodapi_app_with_data_client.get(url_for("events_files|resource"))
        resp_data = json.loads(resp.data.decode("utf-8"))

        for item in resp_data["_items"]:
            assert len(set(item.keys()) & excluded_fields) == 0

        # details
        item = resp_data["_items"][0]
        resp = prodapi_app_with_data_client.get(
            url_for("events_files|item_lookup", _id=item["_id"]),
        )
        resp_data = json.loads(resp.data.decode("utf-8"))

        assert len(set(resp_data.keys()) & excluded_fields) == 0



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
from superdesk.tests import TestCase
from superdesk.io.feed_parsers.pa_nitf import PAFeedParser
from lxml import etree


class PANITFFileTestCase(TestCase):

    vocab = [{"_id": "genre", "items": [{"name": "Current"}]}]

    def setUpForChildren(self):
        super().setUpForChildren()
        with self.app.app_context():
            self.app.data.insert("vocabularies", self.vocab)
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, "../fixtures", self.filename))
        provider = {"name": "Test"}
        with open(fixture, "rb") as f:
            xml = etree.parse(f)
            self.item = PAFeedParser().parse(xml.getroot(), provider)


class PAFileWithNoSubjects(PANITFFileTestCase):

    filename = "pa2.xml"

    def test_headline(self):
        self.assertEqual(self.item.get("headline"), "Soccer INT-Teams")

    def test_anpa_category(self):
        self.assertEqual(self.item.get("anpa_category"), [{"qcode": "S"}])


class PATestCase(PANITFFileTestCase):

    filename = "pa1.xml"

    def test_slugline(self):
        self.assertEqual(self.item.get("slugline"), "Sport Trivia (Oct 14)")
        self.assertEqual(self.item.get("headline"), "PA SPORT TRIVIA (OCTOBER 14)")
        self.assertEqual("usable", self.item.get("pubstatus"))
        self.assertEqual("af1f7ad5-5619-49de-84cc-2e608538c77fSSS-3-1", self.item.get("guid"))
        self.assertEqual(self.item.get("format"), "HTML")
        self.assertEqual(4, len(self.item.get("subject")))
        self.assertIn("Trivia (Oct 14)", self.item.get("keywords"))
        self.assertIsInstance(self.item.get("word_count"), int)


class PAEmbargoTestCase(PANITFFileTestCase):

    filename = "pa3.xml"

    def test_slugline(self):
        self.assertEqual(self.item.get("pubstatus"), "usable")


class PAEntertainmentTest(PANITFFileTestCase):

    filename = "pa4.xml"

    def test_entertainment_category(self):
        self.assertEqual(self.item.get("anpa_category"), [{"qcode": "E"}])


class PACharsetConversionTest(PANITFFileTestCase):

    filename = "pa5.xml"

    def test_charset(self):
        self.assertTrue(
            self.item["body_html"].startswith(
                "<p>Treasury coffers will take a £66 billion annual hit " "if Britain goes for a so-called hard Brexit"
            )
        )
        self.assertEqual(self.item["headline"], "HARD BREXIT TO COST UK UP TO £66BN A YEAR, SAYS TREASURY")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from flask import g, request
from flask_babel import _
from flask_oidc_ex import OpenIDConnect

import superdesk
from apps.auth.errors import CredentialsAuthError
from apps.auth.service import AuthService
from eve.utils import config
from superdesk import get_resource_service
from superdesk.resource import Resource
from superdesk.utils import ignorecase_query

from ..errors import CredentialsAuthError


class OIDCAuthResource(Resource):
    schema = {"token": {"type": "string"}, "user": Resource.rel("users", True)}
    resource_methods = ["POST"]
    public_methods = ["POST"]
    extra_response_fields = ["user", "token", "username"]


superdesk.intrinsic_privilege("auth_oidc", method=["DELETE"])


class OIDCAuthService(AuthService):
    def __init__(self, datasource=None, backend=None, app=None):
        super().__init__(datasource=datasource, backend=backend)
        self.oidc = OpenIDConnect(app)

    def authenticate(self, credentials):
        auth_header = request.headers.get("Authorization", "").split(" ", 1)
        if auth_header[0] != "Bearer" and len(auth_header) != 2:
            raise CredentialsAuthError(credentials)
        token = auth_header[1]
        is_valid = self.oidc.validate_token(token, ["openid", "email", "profile"])
        if not is_valid:
            raise CredentialsAuthError(credentials)

        users_service = get_resource_service("users")
        username = g.oidc_token_info["username"]
        user = users_service.find_one(req=None, username=username) or {}

        sync_data = {
            "username": username,
            "email": g.oidc_token_info.get("email", user.get("email")),
            "display_name": g.oidc_token_info.get("name"),
        }

        # first name and last name is optional in Keycloak
        if "given_name" in g.oidc_token_info:
            sync_data["first_name"] = g.oidc_token_info.get("given_name")
        if "family_name" in g.oidc_token_info:
            sync_data["last_name"] = g.oidc_token_info.get("family_name")

        if not user:
            # email is optional in Keycloak
            if not sync_data["email"]:
                raise CredentialsAuthError(sync_data, message=_("Please update your account email address"))
            user_role = None
            client_id = g.oidc_token_info.get("client_id", "")
            keycloak_roles = g.oidc_token_info.get("resource_access", {}).get(client_id, {}).get("roles", [])
            for role_name in keycloak_roles:
                role = get_resource_service("roles").find_one(req=None, name=ignorecase_query(role_name))
                if role:
                    user_role = role.get("_id")
                    break
            sync_data.update(
                {
                    "password": "",
                    "user_type": "user",
                    "role": user_role,
                    "needs_activation": False,
                }
            )
            users_service.post([sync_data])
        else:
            users_service.patch(user[config.ID_FIELD], sync_data)

        user.update(sync_data)
        return user



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import logging

from superdesk.io.registry import register_feeding_service, register_feeding_service_parser
from superdesk.io.feeding_services.http_base_service import HTTPFeedingServiceBase
from superdesk.errors import IngestApiError, SuperdeskIngestError
from lxml import etree

logger = logging.getLogger(__name__)
URL_ACK = "https://services.ritzau.dk/ritzaurest/Services.svc/xml/news/QueueAcknowledge"


class RitzauFeedingService(HTTPFeedingServiceBase):
    """
    Feeding Service class which can retrieve articles from Ritzau web service
    """

    NAME = "ritzau"

    ERRORS = [
        IngestApiError.apiRequestError().get_error_description(),
        SuperdeskIngestError.notConfiguredError().get_error_description(),
    ]

    label = "Ritzau feed API"

    fields = HTTPFeedingServiceBase.AUTH_FIELDS + [
        {
            "id": "url",
            "type": "text",
            "label": "URL",
            "placeholder": "fill this field only for advanced uses",
            "required": False,
        }
    ]

    HTTP_URL = "https://services.ritzau.dk/ritzaurest/Services.svc/xml/news/NewsQueue"
    # auth is done with params
    HTTP_AUTH = False

    def _update(self, provider, update):
        config = self.config
        try:
            user, password = self.config["username"], self.config["password"]
        except KeyError:
            SuperdeskIngestError.notConfiguredError(Exception("username and password are needed"))

        url_override = config.get("url", "").strip()
        if not url_override.startswith("http"):
            SuperdeskIngestError.notConfiguredError(Exception("if URL is set, it must be a valid http link"))

        if url_override:
            params = {"user": user, "password": password, "maksAntal": 50}
        else:
            params = {"user": user, "password": password, "maksAntal": 50, "waitAcknowledge": "true"}

        r = self.get_url(url_override, params=params)

        try:
            root_elt = etree.fromstring(r.text)
        except Exception:
            raise IngestApiError.apiRequestError(Exception("error while parsing the request answer"))

        try:
            if root_elt.xpath("(//error/text())[1]")[0] != "0":
                err_msg = root_elt.xpath("(//errormsg/text())[1]")[0]
                raise IngestApiError.apiRequestError(Exception("error code returned by API: {msg}".format(msg=err_msg)))
        except IndexError:
            raise IngestApiError.apiRequestError(Exception("Invalid XML, <error> element not found"))

        parser = self.get_feed_parser(provider)
        items = []
        for elt in root_elt.xpath("//RBNews"):
            item = parser.parse(elt, provider)
            items.append(item)
            if not url_override:
                try:
                    queue_id = elt.xpath(".//ServiceQueueId/text()")[0]
                except IndexError:
                    raise IngestApiError.apiRequestError(Exception("missing ServiceQueueId element"))
                ack_params = {"user": user, "password": password, "servicequeueid": queue_id}
                self.get_url(URL_ACK, params=ack_params)

        return [items]


register_feeding_service(RitzauFeedingService)
register_feeding_service_parser(RitzauFeedingService.NAME, "ritzau")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk

from apps.search_providers.registry import (
    registered_search_providers,
    allowed_search_providers,
    register_search_provider,
)  # noqa
from apps.search_providers.resource import SearchProviderResource
from apps.search_providers.service import SearchProviderService
from apps.search_providers.registry import SearchProviderAllowedResource, SearchProviderAllowedService


def init_app(app):
    from apps.search_providers.proxy import SearchProviderProxyResource, SearchProviderProxyService

    superdesk.privilege(
        name="search_providers", label="Manage Search Providers", description="User can manage search providers."
    )

    superdesk.register_resource(name="search_providers", resource=SearchProviderResource, service=SearchProviderService)

    superdesk.register_resource(
        name="search_providers_proxy", resource=SearchProviderProxyResource, service=SearchProviderProxyService
    )

    superdesk.register_resource(
        name="search_providers_allowed", resource=SearchProviderAllowedResource, service=SearchProviderAllowedService
    )



import os
import flask
import unittest


class TestConfig:
    X = True


class ConfigTestCase(unittest.TestCase):
    def test_config_update(self):
        config = flask.Config(os.path.abspath(os.path.dirname(__file__)))
        config.update({"X": True})
        self.assertTrue(config.get("X"))
        config.update({"X": False})
        self.assertFalse(config.get("X"))
        config.from_object(TestConfig())
        self.assertTrue(config.get("X"))



from superdesk import get_resource_service
from superdesk.utc import utcnow
from superdesk.logging import logger
from flask_babel import lazy_gettext


def get_destination_desk(desk, limit=10):
    if not limit or not desk:
        return
    if not desk.get("is_closed"):
        return desk
    if not desk.get("closed_destination"):
        return desk
    return get_destination_desk(
        get_resource_service("desks").find_one(req=None, _id=desk["closed_destination"]), limit - 1
    )


def routing(item, desk=None, **kwargs):
    if desk is None:
        desk_id = item.get("task", {}).get("desk")
        if desk_id:
            desk = get_resource_service("desks").find_one(req=None, _id=desk_id)
    dest = get_destination_desk(desk)
    if dest and str(desk["_id"]) != str(dest["_id"]):
        logger.info(
            'auto-routing item "%s" from desk "%s" to "%s"', item.get("headline"), desk.get("name"), dest.get("name")
        )
        try:
            marked_desks = item.get("marked_desks", [])
            existing = [mark for mark in marked_desks if str(mark["desk_id"]) == str(dest["_id"])]
            if not existing:
                marked_desks.append(
                    {
                        "desk_id": str(dest["_id"]),
                        "date_marked": utcnow(),
                    }
                )
                item["marked_desks"] = marked_desks
        except Exception:
            logger.exception("auto-routing error")
    return item


name = "desk_routing"
label = lazy_gettext("Desk Routing")
callback = routing
access_type = "backend"
action_type = "direct"
